#include <engine/2d/level.h>

/*
 * sct_init:
 *	Initializes the global sct struct with a MOL file.
 *	The MOL file should be initialized with mol_alloc
 *	before calling this function!
 */
sct_init(p_mol, clut_id, tpage_id)
struct AssetMOL *p_mol;
unsigned short clut_id, tpage_id;
{
	int j, n, row = 0, col = 0;
	unsigned int numlayers = p_mol->LYRHEAD.numlayers;

	sct.p_mol = p_mol;

	if(numlayers > SCT_LAYERS_MAX)
	{
		printf("sct_init error: too many layers! (%d/%d)\n", numlayers, SCT_LAYERS_MAX);
		while(1);
	}

	/* initialize layers */
	for(n = 0; n < numlayers; n++)
	{
		struct LYRHEAD_Layer	*layer = &p_mol->LYRHEAD.layer[n];

		for(row = 0; row < SCT_MAX_ROWS; row++)
		{
			for(col = 0; col < SCT_MAX_COLS; col++)
			{
				setSprt16(&sct.layer[n][row][col]);

				setRGB0(&sct.layer[n][row][col], 128, 128, 128);

				sct.layer[n][row][col].x0 = (col * SCT_TILE_SPRITE_SIZE);
				sct.layer[n][row][col].y0 = (row * SCT_TILE_SPRITE_SIZE);

				if(layer->is_sparse)
				{
					unsigned int ntuples = (layer->bnum - MOL_LAYER_METADATA_SIZE)/3;

					for(j = 0; j < ntuples; j++)
					{
						if( (row == layer->p_coo[j].row) && (col == layer->p_coo[j].col) )
						{
							sct.layer[n][row][col].u0 = sct.tileset[layer->p_coo[j].val].u;
							sct.layer[n][row][col].v0 = sct.tileset[layer->p_coo[j].val].v;

						}
					}
				}
				else
				{
					sct.layer[n][row][col].u0 = sct.tileset[layer->p_matrix[row*SCT_MAX_COLS + col].val].u;
					sct.layer[n][row][col].v0 = sct.tileset[layer->p_matrix[row*SCT_MAX_COLS + col].val].v;
				}

				sct.layer[n][row][col].clut = clut_id;
			}
		}
	}

	/* set DR_TPAGE primitive */
	SetDrawTPage(&sct.dr_tpage, 0, 0, tpage_id);
}

/*
 * sct_draw:
 *	Draws the entire section.
 */
sct_draw(x, y)
short x, y;
{
	int n, j;
	int numlayers = sct.p_mol->LYRHEAD.numlayers;

	DrawPrim(&sct.dr_tpage);

	for(n = 0; n < numlayers; n++)
	{
		struct LYRHEAD_Layer *layer = &sct.p_mol->LYRHEAD.layer[n];

		if(layer->is_sparse)
		{
			unsigned int ntuples = (layer->bnum - MOL_LAYER_METADATA_SIZE)/3;
			for(j = 0; j < ntuples; j++)
			{
				unsigned int row = layer->p_coo[j].row;
				unsigned int col = layer->p_coo[j].col;
				short *x0 = &sct.layer[n][row][col].x0;
				short *y0 = &sct.layer[n][row][col].y0;

				*x0 = col*SCT_TILE_SPRITE_SIZE + x;
				*y0 = row*SCT_TILE_SPRITE_SIZE + y;

				if( ((*x0 > -SCT_TILE_SPRITE_SIZE) &&
					(*x0 < SCT_TILES_RENDERED_X*SCT_TILE_SPRITE_SIZE)) )
				{
					if( (*y0 > -SCT_TILE_SPRITE_SIZE) &&
						(*y0 < SCT_TILES_RENDERED_Y*SCT_TILE_SPRITE_SIZE) )
					{
						DrawPrim(&sct.layer[n][row][col]);
					}
				}

			}
		}
		else
		{
			unsigned char	row, col;
			int		leftmost_tile, topmost_tile;

			/* FIXME: should use camera for this, but whatever */
			leftmost_tile	= x;
			topmost_tile	= y;

			/* prevent buffer overflow for horizontal rendering */
			if(leftmost_tile < SCT_RENDER_LEFT_SIDE_LIMIT)
			{
				leftmost_tile = SCT_RENDER_LEFT_SIDE_LIMIT;
			}
			else if(leftmost_tile > SCT_RENDER_RIGHT_SIDE_LIMIT)
			{
				leftmost_tile = SCT_RENDER_RIGHT_SIDE_LIMIT;
			}

			/* prevent buffer overflow for vertical rendering */
			if(topmost_tile < SCT_RENDER_TOP_SIDE_LIMIT)
			{
				topmost_tile = SCT_RENDER_TOP_SIDE_LIMIT;
			}
			else if(topmost_tile > SCT_RENDER_BOTTOM_SIDE_LIMIT)
			{
				topmost_tile = SCT_RENDER_BOTTOM_SIDE_LIMIT;
			}

			/* render */
			for(row = 0; row < SCT_TILES_RENDERED_Y; row++)
			{
				for(col = 0; col < SCT_TILES_RENDERED_X; col++)
				{
					short *x0 = &sct.layer[n][row][col].x0;
					short *y0 = &sct.layer[n][row][col].y0;

					*x0 = col*SCT_TILE_SPRITE_SIZE + x;
					*y0 = row*SCT_TILE_SPRITE_SIZE + y;

					if(layer->p_matrix[row*SCT_MAX_COLS + col].val == SCT_AIR_TILE)
					{
						continue;
					}

					DrawPrim(&sct.layer[n][topmost_tile + row][leftmost_tile + col]);
				}
			}
		}
	}
}

